==> adx-common-2.7.7.1-sources.jar: com/bxm/adx/common/filter/AbstractDspFilter.java <==
package com.bxm.adx.common.filter;

import com.bxm.adx.common.buy.buyers.BuyerWrapper;
import com.bxm.adx.common.buy.dispatcher.Dispatcher;
import com.bxm.adx.common.market.MarketOrders;
import com.bxm.adx.common.market.MarketRequest;
import com.bxm.adx.facade.constant.enums.AdxErrEnum;
import com.bxm.adx.facade.exception.AdxException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.util.ClassUtils;

import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <h3>DSP抽象过滤器</h3>
 *
 * @author hcmony
 * @since V1.0.0, 2020/11/20 16:32
 */

@Slf4j
public abstract class AbstractDspFilter implements Filter {

    public AbstractDspFilter() {
    }

    @Override
    public void filter(FilterRequest request) {

        MarketOrders marketOrders = request.getMarketOrders();
        if (marketOrders == null) {
            log.debug(" marketOrders is null . ");
            return;
        }

        final List<Dispatcher> dsps = doFilter(marketOrders);
        removeDsp(dsps, marketOrders);
        if (CollectionUtils.isEmpty(marketOrders.getRequests())) {
            log.debug("after marketOrders MarketRequest is null . ");
            throw new AdxException(AdxErrEnum.POSITION_FILTER);
        }
    }

    /**
     * DSP相关过滤实现
     *
     * @param marketOrders
     * @return
     */
    protected abstract List<Dispatcher> doFilter(MarketOrders marketOrders);

    /**
     * 移除被过滤掉的DSP
     *
     * @param dispatcherList
     * @param marketOrders
     */
    protected void removeDsp(List<Dispatcher> dispatcherList, MarketOrders marketOrders) {
        if (CollectionUtils.isEmpty(dispatcherList)) {
            return;
        }
        if (log.isDebugEnabled()) {
            String clazzSimpleName = ClassUtils.getUserClass(this.getClass()).getSimpleName();
            log.debug("filter {} remove {}", clazzSimpleName, dispatcherList.stream().map(Dispatcher::getId).collect(Collectors.toSet()));
        }
        List<MarketRequest> requests = marketOrders.getRequests();
        if (CollectionUtils.isEmpty(requests)) {
            return;
        }
        final Iterator<MarketRequest> iterator = requests.iterator();
        dispatcherList.forEach(dispatcher -> {
            while (iterator.hasNext()) {
                final MarketRequest next = iterator.next();
                Iterator<BuyerWrapper> buyerWrapperIterator = next.getBuyers().iterator();
                while (buyerWrapperIterator.hasNext()) {
                    BuyerWrapper buyerWrapper = buyerWrapperIterator.next();
                    Dispatcher currentDispatcher = buyerWrapper.getDispatcher();
                    if (dispatcher.getConfigId().equals(currentDispatcher.getConfigId())
                            && dispatcher.getDspId().equals(currentDispatcher.getDspId()) ) {
                        log.debug("remove dspConfigId = {}, dspId = {}", dispatcher.getConfigId(), dispatcher.getDspId());
                        buyerWrapperIterator.remove();

                        //如果buyers为空，则需要把上层也移除
                        if (next.getBuyers().size() == 0) {
                            iterator.remove();
                        }
                    }
                }
            }

        });
    }
}

==> adx-common-2.7.9.24-sources.jar: com/bxm/adx/common/filter/AbstractDspFilter.java <==
package com.bxm.adx.common.filter;

import com.bxm.adx.common.buy.buyers.BuyerWrapper;
import com.bxm.adx.common.buy.dispatcher.Dispatcher;
import com.bxm.adx.common.market.MarketOrders;
import com.bxm.adx.common.market.MarketRequest;
import com.bxm.adx.facade.constant.enums.AdxErrEnum;
import com.bxm.adx.facade.exception.AdxException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.util.ClassUtils;

import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <h3>DSP抽象过滤器</h3>
 *
 * @author hcmony
 * @since V1.0.0, 2020/11/20 16:32
 */

@Slf4j
public abstract class AbstractDspFilter implements Filter {

    public AbstractDspFilter() {
    }

    @Override
    public void filter(FilterRequest request) {

        MarketOrders marketOrders = request.getMarketOrders();
        if (marketOrders == null) {
            log.debug(" marketOrders is null . ");
            return;
        }

        final List<Dispatcher> dsps = doFilter(marketOrders);
        removeDsp(dsps, marketOrders);
        if (CollectionUtils.isEmpty(marketOrders.getRequests())) {
            log.debug("after marketOrders MarketRequest is null . ");
            throw new AdxException(AdxErrEnum.POSITION_FILTER);
        }
    }

    /**
     * DSP相关过滤实现
     *
     * @param marketOrders
     * @return
     */
    protected abstract List<Dispatcher> doFilter(MarketOrders marketOrders);

    /**
     * 移除被过滤掉的DSP
     *
     * @param dispatcherList
     * @param marketOrders
     */
    protected void removeDsp(List<Dispatcher> dispatcherList, MarketOrders marketOrders) {
        if (CollectionUtils.isEmpty(dispatcherList)) {
            return;
        }
        if (log.isDebugEnabled()) {
            String clazzSimpleName = ClassUtils.getUserClass(this.getClass()).getSimpleName();
            log.debug("filter {} remove {}", clazzSimpleName, dispatcherList.stream().map(Dispatcher::getId).collect(Collectors.toSet()));
        }
        List<MarketRequest> requests = marketOrders.getRequests();
        if (CollectionUtils.isEmpty(requests)) {
            return;
        }
        dispatcherList.forEach(dispatcher -> {
            final Iterator<MarketRequest> iterator = requests.iterator();
            while (iterator.hasNext()) {
                final MarketRequest next = iterator.next();
                Iterator<BuyerWrapper> buyerWrapperIterator = next.getBuyers().iterator();
                while (buyerWrapperIterator.hasNext()) {
                    BuyerWrapper buyerWrapper = buyerWrapperIterator.next();
                    Dispatcher currentDispatcher = buyerWrapper.getDispatcher();
                    if (dispatcher.getConfigId().equals(currentDispatcher.getConfigId())
                            && dispatcher.getDspId().equals(currentDispatcher.getDspId())
                            && dispatcher.getDspPosid().equals(currentDispatcher.getDspPosid()) ) {
                        log.debug("remove dspConfigId = {}, dspId = {}", dispatcher.getConfigId(), dispatcher.getDspId());
                        buyerWrapperIterator.remove();
                    }
                }
                //如果buyers为空，则需要把上层也移除
                if (next.getBuyers().size() == 0) {
                    iterator.remove();
                }
            }

        });
    }
}
